@@ -3,7 +3,7 @@ package Perl6::Perldoc::To::Ansi;
 use warnings;
 use strict;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 # add fake opening/closing tags, to be processed later
 sub add_ansi {
@@ -11,6 +11,13 @@ sub add_ansi {
     return "\e[OPEN${new}m" . $text . "\e[CLOSE${new}m";
 }
 
+# same, but only if the entire text is not already colored
+sub add_ansi_only {
+    my ($text, $new) = @_;
+    return $text if $text =~ /^\e\[/ && $text =~ /\e\[0?m$/;
+    return add_ansi($text, $new);
+}
+
 sub rewrite_ansi {
     my ($text) = @_;
     #$text = "\e[${new}m$text";
@@ -213,7 +220,7 @@ sub to_ansi {
     if (defined $number) {
         $title = "$number. $title";
     }
-    return "\n\n" . Perl6::Perldoc::To::Ansi::add_ansi(uc $title, '1') ."\n";
+    return "\n\n" . Perl6::Perldoc::To::Ansi::add_ansi_only($title, '1') ."\n";
 }
 
 # Standard =head2 block...
@@ -228,7 +235,7 @@ sub to_ansi {
     if (defined $number) {
         $title = "$number. $title";
     }
-    return "\n\n" . Perl6::Perldoc::To::Ansi::add_ansi($title, '1') ."\n";
+    return "\n\n" . Perl6::Perldoc::To::Ansi::add_ansi_only($title, '1') ."\n";
 }
 
 # Standard =head3 block...
@@ -243,7 +250,7 @@ sub to_ansi {
     if (defined $number) {
         $title = "$number. $title";
     }
-    return "\n\n$title\n";
+    return "\n\n" . Perl6::Perldoc::To::Ansi::add_ansi_only($title, '1') ."\n";
 }
 
 # Standard =head4 block...
@@ -258,7 +265,7 @@ sub to_ansi {
     if (defined $number) {
         $title = "$number. $title";
     }
-    return "\n\n$title\n";
+    return "\n\n" . Perl6::Perldoc::To::Ansi::add_ansi_only($title, '1') ."\n";
 }
 
 # Implicit list block...
@@ -526,35 +533,43 @@ my $SEARCH      = 'http://www.google.com/search?q=';
 
 sub to_ansi {
     my $self = shift;
-    my $target = Perl6::Perldoc::To::Ansi::add_ansi($self->target(), '34');
+    my $target = $self->target();
     my $text = $self->has_distinct_text ? $self->SUPER::to_ansi(@_) : undef;
-
+    my $add_color = sub {
+        $target = Perl6::Perldoc::To::Ansi::add_ansi($target, '34');
+    };
+    
     # Link within this document...
     if ($target =~ s{\A (?:doc:\s*)? [#] }{}xms ) {
-        return defined $text ? qq{$text (see the "$target" section)}
-                             : qq{the "$target" section}
+        $add_color->();
+        return defined $text ? qq{$text (see the $target section)}
+                             : qq{the $target section}
     }
 
     # Link to other documentation...
     if ($target =~ s{\A doc: }{}xms) {
+        $add_color->();
         return defined $text ? qq{$text (see the documentation for $target)} 
                              : qq{the documentation for $target}
     }
 
     # Link to manpage...
     if ($target =~ s{\A man: }{}xms) {
+        $add_color->();
         return defined $text ? qq{$text (see the $target manpage)}
                              : qq{the $target manpage}
     }
 
     # Link back to definition in this document...
     if ($target =~ s{\A (?:defn) : }{}xms) {
-        return defined $text ? qq{$text (see the definition of "$target")}
+        $add_color->();
+        return defined $text ? qq{$text (see the definition of $target)}
                              : $target
     }
 
     # Anything else...
-    return defined $text ? qq{$text <$target>}
+    $add_color->();
+    return defined $text ? qq{$text $target}
                          : $target;
 }
 
